# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for manipulating a set of disks."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base

DETAILED_HELP = {
    'DESCRIPTION':
        """
      Manipulate multiple Compute Engine disks with single command executions.
    """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Bulk(base.Group):
  """Manipulate multiple Compute Engine disks with single command executions."""


Bulk.detailed_help = DETAILED_HELP


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class BulkBeta(Bulk):
  """Manipulate multiple Compute Engine disks with single command executions."""


BulkBeta.detailed_help = DETAILED_HELP


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class BulkAlpha(Bulk):
  """Manipulate multiple Compute Engine disks with single command executions."""


BulkAlpha.detailed_help = DETAILED_HELP
