# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command group for cloud dataproc session templates."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class SessionTemplates(base.Group):
  """Create and manage Dataproc session templates.

  Create and manage Dataproc session templates.

  ## EXAMPLES

  To see the list of all session templates, run:

    $ {command} list

  To view the details of a session template, run:

    $ {command} describe my-template

  To view just the non-output only fields of a session template, run:

    $ {command} export my-template --destination template-file.yaml

  To create or update a session template, run:

    $ {command} import my-template --source template-file.yaml

  To delete a session template, run:

    $ {command} delete my-template
  """
  pass
