# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the cloud deploy custom target type resource."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.clouddeploy import custom_target_type


def PatchCustomTargetType(resource):
  """Patches a custom target type resource.

  Args:
    resource: apitools.base.protorpclite.messages.Message, custom target type
      message.

  Returns:
    The operation message
  """
  return custom_target_type.CustomTargetTypesClient().Patch(resource)


def DeleteCustomTargetType(name):
  """Deletes a custom target type resource.

  Args:
    name: str, custom target type name.

  Returns:
    The operation message
  """
  return custom_target_type.CustomTargetTypesClient().Delete(name)
